/*
 * $Id: render.c,v 1.1.1.1 2002/03/07 15:33:14 kenta Exp $
 *
 * Copyright 2001 Kenta Cho. All rights reserved.
 */

/**
 * Render 3D wire.
 *
 * @version $Revision: 1.1.1.1 $
 */
#include <stdlib.h>
#include <piece.h>

#include "sbrg.h"
#include "render.h"
#include "vector.h"
#include "sincos.h"

#define POINT_MAX 269
#define LINE_MAX 196
#define DOT_MAX 69

static Point point[POINT_MAX];
static Line line[LINE_MAX];
static Dot dot[DOT_MAX];

static int pointIdx, lineIdx, dotIdx;

#define NOT_EXIST -999999999

// Handle ring buffers for Point/Line/Dot.

void initRender() {
  int i;
  for ( i=0 ; i<POINT_MAX ; i++ ) {
    point[i].a.x = NOT_EXIST;
  }
  pointIdx = -1;//POINT_MAX;
  for ( i=0 ; i<LINE_MAX ; i++ ) {
    //line[i].sp = NULL;
    line[i].color = NOT_EXIST;
  }
  lineIdx = -1;//LINE_MAX;
  for ( i=0 ; i<DOT_MAX ; i++ ) {
    //dot[i].p = NULL;
    dot[i].size = NOT_EXIST;
  }
  dotIdx = -1;//DOT_MAX;
}

Point* getPointInstance() {
  /*int i;
  for ( i=0 ; i<POINT_MAX ; i++ ) {
    pointIdx--; if ( pointIdx < 0 ) pointIdx = POINT_MAX-1;
    if ( point[pointIdx].a.x == NOT_EXIST ) {
      return &(point[pointIdx]);
    }
  }
  return NULL;*/
  pointIdx++;
  if ( pointIdx >= POINT_MAX ) pceAppReqExit(1);
  return &(point[pointIdx]);
}

Line* getLineInstance() {
  /*int i;
  for ( i=0 ; i<LINE_MAX ; i++ ) {
    lineIdx--; if ( lineIdx < 0 ) lineIdx = LINE_MAX-1;
    if ( line[lineIdx].sp == NULL ) {
      return &(line[lineIdx]);
    }
  }
  return NULL;*/
  lineIdx++;
  if ( lineIdx >= POINT_MAX ) pceAppReqExit(1);
  return &(line[lineIdx]);
}

Dot* getDotInstance() {
  /*int i;
  for ( i=0 ; i<DOT_MAX ; i++ ) {
    dotIdx--; if ( dotIdx < 0 ) dotIdx = DOT_MAX-1;
    if ( dot[dotIdx].p == NULL ) {
      return &(dot[dotIdx]);
    }
  }
  return NULL;*/
  dotIdx++;
  if ( dotIdx >= POINT_MAX ) pceAppReqExit(1);
  return &(dot[dotIdx]);
}


#define RENDER_CENTER_X (SCREEN_WIDTH>>1)
#define RENDER_CENTER_Y (-(SCREEN_HEIGHT>>1))

void renderAll(Vector *ofs, int d1) {
  int i;
  int x, y, z, tx;
  Point *p;
  Line *l;
  Dot *d;
  int oz, dz;
  int x1, y1, x2, y2;
  int eyeSin = sctbl[d1], eyeCos = sctbl[d1+64];
  int dSize;

  // Map absolute positions to relative/mapped positions.
  //for ( i=0 ; i<POINT_MAX ; i++ ) {
  for ( i=0 ; i<=pointIdx ; i++ ) {
    if ( point[i].a.x == NOT_EXIST ) continue;
    p = &(point[i]);
    tx = p->a.x - ofs->x;
    y  = p->r.y = ofs->y - p->a.y; //-(p->a.y - ofs->y);
    z  = p->a.z - ofs->z;
    x = p->r.x = (tx*eyeCos - z*eyeSin)>>8;
    z = p->r.z = (tx*eyeSin + z*eyeCos)>>8;
    if ( z > DEPTH ) {
      p->m.x = (x<<8)/z + RENDER_CENTER_X;
      p->m.y = (y<<8)/z + RENDER_CENTER_Y;
      p->m.z = 0;
    } else {
      p->m.z = NOT_EXIST;
    }
  }

  // Draw lines.
  //for ( i=0 ; i<LINE_MAX ; i++ ) {
  for ( i=0 ; i<=lineIdx ; i++ ) {
    //if ( line[i].sp == NULL ) continue;
    if ( line[i].color == NOT_EXIST ) continue;
    l = &(line[i]);
    if ( l->sp->m.z == NOT_EXIST ) {
      if ( l->ep->m.z == NOT_EXIST ) continue;
      oz = l->ep->r.z - l->sp->r.z;
      dz = DEPTH - l->sp->r.z;
      x1 = ((l->ep->r.x - l->sp->r.x)*dz) / oz + l->sp->r.x + RENDER_CENTER_X;
      y1 = ((l->ep->r.y - l->sp->r.y)*dz) / oz + l->sp->r.y + RENDER_CENTER_Y;
      x2 = l->ep->m.x;
      y2 = l->ep->m.y;
    } else {
      if ( l->ep->m.z == NOT_EXIST ) {
	oz = l->sp->r.z - l->ep->r.z;
	dz = DEPTH - l->ep->r.z;
	x2 = ((l->sp->r.x - l->ep->r.x)*dz) / oz + l->ep->r.x + RENDER_CENTER_X;
	y2 = ((l->sp->r.y - l->ep->r.y)*dz) / oz + l->ep->r.y + RENDER_CENTER_Y;
	x1 = l->sp->m.x;
	y1 = l->sp->m.y;
      } else {
	x1 = l->sp->m.x;
	y1 = l->sp->m.y;
	x2 = l->ep->m.x;
	y2 = l->ep->m.y;
      }
    }
    if ( x1 < x2 ) {
      if ( x1 < 0 ) {
	if ( x2 < 0 ) continue;
	y1 = (y2-y1)*-x1/(x2-x1) + y1;
	x1 = 0;
      }
      if ( x2 >= SCREEN_WIDTH ) {
	if ( x1 >= SCREEN_WIDTH ) continue;
	y2 = (y1-y2)*(SCREEN_WIDTH-x2)/(x1-x2) + y2;
	x2 = SCREEN_WIDTH-1;
      }
    } else {
      if ( x2 < 0 ) {
	if ( x1 < 0 ) continue;
	y2 = (y1-y2)*-x2/(x1-x2) + y2;
	x2 = 0;
      }
      if ( x1 >= SCREEN_WIDTH ) {
	if ( x2 >= SCREEN_WIDTH ) continue;
	y1 = (y2-y1)*(SCREEN_WIDTH-x1)/(x2-x1) + y1;
	x1 = SCREEN_WIDTH-1;
      }
    }
    if ( y1 < y2 ) {
      if ( y1 < 0 ) {
	if ( y2 < 0 ) continue;
	x1 = (x2-x1)*-y1/(y2-y1) + x1;
	y1 = 0;
      }
      if ( y2 >= SCREEN_HEIGHT ) {
	if ( y1 >= SCREEN_HEIGHT ) continue;
	x2 = (x1-x2)*(SCREEN_HEIGHT-y2)/(y1-y2) + x2;
	y2 = SCREEN_HEIGHT-1;
      }
    } else {
      if ( y2 < 0 ) {
	if ( y1 < 0 ) continue;
	x2 = (x1-x2)*-y2/(y1-y2) + x2;
	y2 = 0;
      }
      if ( y1 >= SCREEN_HEIGHT ) {
	if ( y2 >= SCREEN_HEIGHT ) continue;
	x1 = (x2-x1)*(SCREEN_HEIGHT-y1)/(y2-y1) + x1;
	y1 = SCREEN_HEIGHT-1;
      }
    }
    pceLCDLine(l->color, x1, y1, x2, y2);
  }

  // Draw dots.
  //for ( i=0 ; i<DOT_MAX ; i++ ) {
  for ( i=0 ; i<=dotIdx ; i++ ) {
    //if ( dot[i].p == NULL || dot[i].p->m.z == NOT_EXIST ) continue;
    if ( dot[i].size == NOT_EXIST || dot[i].p->m.z == NOT_EXIST ) continue;
    d = &(dot[i]);
    dSize = d->size;
    pceLCDPaint(d->color, d->p->m.x-(dSize>>1), d->p->m.y-(dSize>>1), dSize, dSize);
  }
}
