/*
 * $Id: sbrg.c,v 1.3 2002/03/13 16:09:58 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Speed Barricade bootstrap.
 *
 * @version $Revision: 1.3 $
 */
#include <string.h>
#include <stdlib.h>
#include <piece.h>

#include "sbrg.h"
#include "gamemanager.h"
#include "render.h"
#include "usb.h"

unsigned char vbuf[SCREEN_WIDTH*SCREEN_HEIGHT];

// sounds
static PCEWAVEINFO wtemp;
static char *sndBuf[4];
static int prvAtt[4];

// images
extern unsigned char IMG_TITLE[], IMG_START[], IMG_BEAT[], IMG_OPP[],
  IMG_ZERO[], IMG_ONE[], IMG_TWO[], IMG_THREE[], IMG_FOUR[],
  IMG_FIVE[], IMG_SIX[], IMG_SEVEN[], IMG_EIGHT[], IMG_NINE[];
PIECE_BMP titleBmp, startBmp, beatBmp, oppBmp, numBmp[10];

int status;
int cnt = 0;

void initDemo() {
  status = DEMO; cnt = -1;
  initGameDemo();
}

void initInGame() {
  status = IN_GAME; cnt = -1;
  initGame();
}

static void loadBmp(unsigned char *data, PIECE_BMP *bmp) {
  PBMP_FILEHEADER pbhead;
  memcpy(&pbhead, data, sizeof(PBMP_FILEHEADER));
  bmp->header = pbhead;
  bmp->buf  = data + sizeof(PBMP_FILEHEADER);
  bmp->mask = data + sizeof(PBMP_FILEHEADER) + (bmp->header.h*bmp->header.w)/4;
}

// Handle sounds.

void playSe(char *snd, int ch) {
  sndBuf[ch] = snd;
}

static void stopSe() {
  int i;
  for ( i=0 ; i<4 ; i++ ) {
    pceWaveAbort(i);
    sndBuf[i] = NULL;
  }
}

static void playAllSound() {
  int i;
  for ( i=0 ; i<4 ; i++ ) {
    if ( sndBuf[i] == NULL ) continue;
    wtemp = *((PCEWAVEINFO*)(sndBuf[i] + 8));
    wtemp.pData = sndBuf[i] + 8 + sizeof(PCEWAVEINFO);
    pceWaveDataOut(i, &wtemp);
    sndBuf[i] = NULL;
  }
}

static void saveWaveAtt() {
  int i;
  for ( i=0 ; i<4 ; i++ ) {
    prvAtt[i] = pceWaveSetChAtt(i, INVALIDVAL);
  }
}

static void loadWaveAtt() {
  int i;
  for ( i=0 ; i<4 ; i++ ) {
    pceWaveSetChAtt(i, prvAtt[i]);
  }
}


static unsigned long seCount, bgmCount;
int playBgmTick;

#define INTERVAL 31

void pceAppInit() {
  // Randomize.
  PCETIME time;
  pceTimeGet(&time);
  srand(time.s100 + time.ss*100);

  // No pad auto repeat.
  while(pcePadGet()) pcePadGetProc();
  pcePadSetTrigMode(PP_MODE_SINGLE);

  pceLCDDispStop();
  pceLCDSetBuffer(vbuf);

  sndBuf[0] = sndBuf[1] = sndBuf[2] = sndBuf[3] = NULL;
  seCount = bgmCount = 0;
  playBgmTick = 0;
  saveWaveAtt();

  loadBmp(IMG_TITLE, &titleBmp);
  loadBmp(IMG_START, &startBmp);
  loadBmp(IMG_BEAT, &beatBmp);
  loadBmp(IMG_OPP, &oppBmp);
  loadBmp(IMG_ZERO, &numBmp[0]);
  loadBmp(IMG_ONE, &numBmp[1]);
  loadBmp(IMG_TWO, &numBmp[2]);
  loadBmp(IMG_THREE, &numBmp[3]);
  loadBmp(IMG_FOUR, &numBmp[4]);
  loadBmp(IMG_FIVE, &numBmp[5]);
  loadBmp(IMG_SIX, &numBmp[6]);
  loadBmp(IMG_SEVEN, &numBmp[7]);
  loadBmp(IMG_EIGHT, &numBmp[8]);
  loadBmp(IMG_NINE, &numBmp[9]);

  initUSB();
  initRender();
  initGamemanager();

  initDemo();

  pceAppSetProcPeriod(INTERVAL);
  pceLCDDispStart();
}

void clearScreen() {
  memset(vbuf, 0, SCREEN_WIDTH*SCREEN_HEIGHT);
}

void pceAppProc(int c) {
  int pad = pcePadGet();

  // Play SE/BGM at this timing.
  unsigned long tc = pceTimerGetCount();
  if ( tc-seCount > INTERVAL*2+INTERVAL/4 ) {
    playAllSound();
    seCount = tc;
  }
  if ( tc-bgmCount > INTERVAL*47+INTERVAL/4 ) {
    playBgmTick = 1;
    bgmCount = tc;
    seCount = tc+9999;
  } else {
    playBgmTick = 0;
  }

  cnt++;

  switch ( status ) {
  case DEMO:
    // Start game when TRG_C(start) is pressed.
    if ( pad & TRG_C ) {
      initInGame();
      return;
    }
    moveEye(pad);
  case IN_GAME:
    clearScreen();
    moveMotors(pad);
    moveFrags();
    drawScene(pad);
    break;
  }

  // Exit application when TRG_D(select) is pressed.
  if ( pad & TRG_D ) {
    pceAppReqExit(1);
  }
  pceLCDTrans();
}

void pceAppExit( void ) {
  closeUSB();
  stopSe();
  loadWaveAtt();
}
