/*
 * $Id: usb.c,v 1.1 2002/03/13 16:10:29 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * USB handler.
 *
 * @version $Revision: 1.1 $
 */
#include <string.h>
#include <piece.h>

#include "usb.h"

#define USB_SEND_RING_SIZE 32
#define USB_SEND_BUF_SIZE 16

static unsigned char sendDataBuf[USB_SEND_RING_SIZE][USB_SEND_BUF_SIZE];
static unsigned char sendDataBufSize[USB_SEND_RING_SIZE];
static int bufSendIdx = 0;
static int bufSetIdx = 0;

void initUSB() {
  USBCOMINFO usbInfo;
  int i;

  pceUSBSetupMode(PUM_BASIC, NULL, NULL);

  strcpy(usbInfo.signature, "Sbrg");
  pceUSBCOMSetup(&usbInfo);

  for ( i=0 ; i<USB_SEND_RING_SIZE ; i++ ) {
    sendDataBufSize[i] = 0;
  }
}

void closeUSB() {
  int i;
  for ( i=0 ; i<32 ; i++ ) {
    if ( pceUSBCOMStop() == 0 ) break;
  }
}

// Send one data per one frame.
void flushOneSendDataBuf() {
  if ( (pceUSBCOMGetStat() & (UCS_TXING | UCS_TXWAIT)) != 0 ) return;
  if ( sendDataBufSize[bufSetIdx] > 0 ) {
    bufSetIdx++; bufSetIdx &= (USB_SEND_RING_SIZE-1);
    sendDataBufSize[bufSetIdx] = 0;
  }
  if ( bufSendIdx == bufSetIdx ) return;
  pceUSBCOMStartTx(sendDataBuf[bufSendIdx], sendDataBufSize[bufSendIdx]);
  bufSendIdx++; bufSendIdx &= (USB_SEND_RING_SIZE-1);
}

// Set a command to data buffer.

// Set motor initial potision.
#define SET_COMMAND 0
// Update motor position.
#define UPDATE_COMMAND 1
// Wall destroied.
#define DESTROY_COMMAND 2
// Motor destroied.
#define DEAD_COMMAND 3
// Player motor crashed into wall.
#define CRASH_COMMAND 4
// Reset game.
#define RESET_COMMAND 5

#define XYD_DATA_SIZE 1+1+1+2+2+1
#define XY_DATA_SIZE 1+1+2+2
#define IDX_DATA_SIZE 1+1+1
#define CMD_DATA_SIZE 1

static void setDataXYD(int cmd, int motorIdx, int wallIdx, int x, int y, int d) {
  unsigned char *pd;
  if ( sendDataBufSize[bufSetIdx] + XYD_DATA_SIZE > USB_SEND_BUF_SIZE ) {
    bufSetIdx++; bufSetIdx &= (USB_SEND_RING_SIZE-1);
    sendDataBufSize[bufSetIdx] = 0;
  }
  pd = &(sendDataBuf[bufSetIdx][sendDataBufSize[bufSetIdx]]);
  *pd = (unsigned char)cmd; pd++;
  *pd = (unsigned char)motorIdx; pd++;
  *pd = (unsigned char)wallIdx; pd++;
  *pd = (unsigned char)((x&0xff00)>>8); pd++;
  *pd = (unsigned char)(x&0xff); pd++;
  *pd = (unsigned char)((y&0xff00)>>8); pd++;
  *pd = (unsigned char)(y&0xff); pd++;
  *pd = (unsigned char)d;
  sendDataBufSize[bufSetIdx] += XYD_DATA_SIZE;
}

static void setDataXY(int cmd, int motorIdx, int x, int y) {
  unsigned char *pd;
  if ( sendDataBufSize[bufSetIdx] + XY_DATA_SIZE > USB_SEND_BUF_SIZE ) {
    bufSetIdx++; bufSetIdx &= (USB_SEND_RING_SIZE-1);
    sendDataBufSize[bufSetIdx] = 0;
  }
  pd = &(sendDataBuf[bufSetIdx][sendDataBufSize[bufSetIdx]]);
  *pd = (unsigned char)cmd; pd++;
  *pd = (unsigned char)motorIdx; pd++;
  *pd = (unsigned char)((x&0xff00)>>8); pd++;
  *pd = (unsigned char)(x&0xff); pd++;
  *pd = (unsigned char)((y&0xff00)>>8); pd++;
  *pd = (unsigned char)(y&0xff);
  sendDataBufSize[bufSetIdx] += XY_DATA_SIZE;
}

static void setDataIdx(int cmd, int motorIdx, int wallIdx) {
  unsigned char *pd;
  if ( sendDataBufSize[bufSetIdx] + IDX_DATA_SIZE > USB_SEND_BUF_SIZE ) {
    bufSetIdx++; bufSetIdx &= (USB_SEND_RING_SIZE-1);
    sendDataBufSize[bufSetIdx] = 0;
  }
  pd = &(sendDataBuf[bufSetIdx][sendDataBufSize[bufSetIdx]]);
  *pd = (unsigned char)cmd; pd++;
  *pd = (unsigned char)motorIdx; pd++;
  *pd = (unsigned char)wallIdx;
  sendDataBufSize[bufSetIdx] += IDX_DATA_SIZE;
}

static void setDataCmd(int cmd) {
  unsigned char *pd;
  if ( sendDataBufSize[bufSetIdx] + CMD_DATA_SIZE > USB_SEND_BUF_SIZE ) {
    bufSetIdx++; bufSetIdx &= (USB_SEND_RING_SIZE-1);
    sendDataBufSize[bufSetIdx] = 0;
  }
  pd = &(sendDataBuf[bufSetIdx][sendDataBufSize[bufSetIdx]]);
  *pd = (unsigned char)cmd;
  sendDataBufSize[bufSetIdx] += CMD_DATA_SIZE;
}

void setWallUSB(int motorIdx, int wallIdx, int x, int y, int d) {
  setDataXYD(SET_COMMAND, motorIdx, wallIdx, x, y, d);
}

void updateWallUSB(int motorIdx, int wallIdx, int x, int y, int d) {
  setDataXYD(UPDATE_COMMAND, motorIdx, wallIdx, x, y, d);
}

void destroyWallUSB(int motorIdx, int wallIdx) {
  setDataIdx(DESTROY_COMMAND, motorIdx, wallIdx);
}

void deadMotorUSB(int motorIdx, int x, int y) {
  setDataXY(DEAD_COMMAND, motorIdx, x, y);
}

void crashMotorUSB(int motorIdx, int x, int y) {
  setDataXY(CRASH_COMMAND, motorIdx, x, y);
}

void resetGameUSB() {
  setDataCmd(RESET_COMMAND);
}
